import { parsePattern, createRouteRegex, getVariables, logger } from './helpers';
export class RouterInternal {
    constructor() {
        this.active = [];
        this.routers = [];
        this.routersConfig = [];
        this.usehash = true;
        this.hashChange = this.hashChange.bind(this);
        window.addEventListener('hashchange', this.hashChange);
    }
    activateRouterElement(name) {
        this.active.push(name);
        if (this.loadingHandlerCallback) {
            this.loadingHandlerCallback(location.hash, this.active[this.active.length - 1]);
        }
        this.dowork().then(() => {
            if (this.foundRoute === false) {
                this.routeNotFound();
            }
            if (this.routeChangeHandlerCallback) {
                this.routeChangeHandlerCallback(this.activeRoute, this.active[this.active.length - 1]);
            }
        });
    }
    deactivateRouterElement(name) {
        let i = this.active.indexOf(name);
        if (i !== -1) {
            this.active.splice(i, 1);
        }
    }
    goto(hash, params = {}) {
        const urls = hash.split('/').filter(x => (x ? true : false));
        let newUrl = '';
        urls.forEach((val, i) => {
            if (val[0] === ':' && params[val.substr(1, val.length)] !== undefined) {
                newUrl = newUrl + params[val.substr(1, val.length)];
            }
            else {
                newUrl = newUrl + `${val}`;
            }
            if (urls.length - 1 !== i) {
                newUrl = newUrl + `/`;
            }
        });
        location.hash = newUrl;
    }
    getNavLinks(routerName, active) {
        let result = [];
        let r_config = this.routersConfig[this.routers.indexOf(active ? this.active[this.active.length - 1] : routerName)];
        r_config.forEach(r => {
            result.push({
                name: r.name,
                isNav: r.isNav === false ? false : true,
                href: (this.usehash ? '#' : '') + r.path,
                data: r.data,
                componentName: r.componentName,
                isAuth: r.isAuth
            });
        });
        return result;
    }
    getNavLink(pathname, routerName, active) {
        let result = {};
        let r_config = this.routersConfig[this.routers.indexOf(active ? this.active[this.active.length - 1] : routerName)];
        r_config.forEach(r => {
            if (r.name === pathname) {
                result = {
                    name: r.name,
                    isNav: r.isNav === false ? false : true,
                    href: (this.usehash ? '#' : '') + r.path,
                    data: r.data,
                    componentName: r.componentName,
                    isAuth: r.isAuth
                };
            }
        });
        return result;
    }
    async hashChange() {
        this.lastpop = null;
        logger('router-hashChange', 'start', this.active[this.active.length - 1]);
        if (this.loadingHandlerCallback) {
            this.loadingHandlerCallback(location.hash, this.active[this.active.length - 1]);
        }
        if (this.backEventtriggered) {
            this.backEventtriggered = false;
        }
        else {
            await this.dowork();
            if (this.foundRoute === false) {
                if (this.active.length > 1) {
                    let y = this.active.pop();
                    if (y !== this.lastpop) {
                        this.lastpop = y;
                        await this.dowork();
                    }
                }
                if (this.active.length === 1 && !this.foundRoute) {
                    this.routeNotFound();
                }
            }
            if (this.routeChangeHandlerCallback) {
                this.routeChangeHandlerCallback(this.activeRoute, this.active[this.active.length - 1]);
            }
        }
    }
    async dowork() {
        this.foundRoute = null;
        let hash = location.hash ? location.hash.substring(1, location.hash.length) : '';
        let routerElements = document.getElementsByTagName('free-router');
        let ok = true;
        for (let i = 0; routerElements.length > i; i++) {
            if (routerElements[i].getAttribute('name') === this.active[this.active.length - 1]) {
                let firstchild = routerElements[i].children[0];
                if (firstchild && firstchild.canDeactivate) {
                    ok = await firstchild.canDeactivate();
                    if (ok !== true) {
                        this.backEventtriggered = true;
                        history.back();
                    }
                }
            }
        }
        if (ok === true) {
            this.foundRoute = false;
            logger('router-hashChange', 'routeSearch', this.active[this.active.length - 1]);
            for (let i = 0; routerElements.length > i; i++) {
                if (!this.foundRoute && routerElements[i].getAttribute('name') === this.active[this.active.length - 1]) {
                    let r_config = this.routersConfig[this.routers.indexOf(this.active[this.active.length - 1])];
                    if (r_config) {
                        for (let y = 0; y < r_config.length; y++) {
                            const route = r_config[y];
                            let verified = false;
                            if (hash === '' || route.path === '') {
                                verified = hash === route.path;
                            }
                            else {
                                let regex = new RegExp(createRouteRegex(parsePattern(route.path), route.children));
                                if (regex.test(hash)) {
                                    verified = true;
                                }
                            }
                            if (verified && route.isAuth) {
                                if (!this.routeAuth(this.getNavLink(route.name, this.active[this.active.length - 1]))) {
                                    this.foundRoute = true;
                                    verified = false;
                                    y = r_config.length;
                                }
                            }
                            if (verified) {
                                y = r_config.length;
                                this.activeRoute = {
                                    name: route.name,
                                    href: route.path,
                                    isNav: route.isNav ? true : false,
                                    data: Object.assign({}, route.data)
                                };
                                logger('router-hashChange', 'verified', this.active[this.active.length - 1], hash, route.name);
                                this.foundRoute = true;
                                if (route.load) {
                                    await route.load();
                                }
                                if (routerElements[i].children.length) {
                                    logger('router-hashChange', 'removing child', this.active[this.active.length - 1], hash);
                                    routerElements[i].removeChild(routerElements[i].children[0]);
                                }
                                let el = document.createElement(route.componentName.toUpperCase());
                                logger('router-hashChange', 'chreate element', this.active[this.active.length - 1], hash);
                                if (this.haltedActivate) {
                                    this.haltedActivate = null;
                                }
                                if (el.activate) {
                                    let active = this.active[this.active.length - 1];
                                    this.haltedActivate = route.path;
                                    await el.activate(route.path, getVariables(parsePattern(route.path), hash), this.activeRoute);
                                    if (this.haltedActivate === route.path) {
                                        logger('router-hashChange', 'append child', this.active[this.active.length - 1], hash);
                                        routerElements[i].appendChild(el);
                                    }
                                    else {
                                        el.disconnectedCallback ? el.disconnectedCallback() : null;
                                        logger('router-hashChange', 'skipping append child', active, hash);
                                    }
                                }
                                else {
                                    logger('router-hashChange', 'append child', this.active[this.active.length - 1], hash);
                                    routerElements[i].appendChild(el);
                                }
                            }
                            else {
                                logger('router-hashChange', 'not-verified', this.active[this.active.length - 1], hash, route.name);
                            }
                        }
                    }
                }
            }
        }
    }
    routeNotFound() {
        if (this.routeNotFoundCallback) {
            this.routeNotFoundCallback(location.hash);
        }
        else {
            console.log('route not found', location.hash);
        }
    }
    registerUnknowRouteHandler(callback) {
        this.routeNotFoundCallback = callback;
    }
    routeAuth(options) {
        if (this.routeAuthCallback) {
            return this.routeAuthCallback(options);
        }
        else {
            console.log('no auth function registered', location.hash);
            return true;
        }
    }
    href(name, _router) {
        let result = 'unknown';
        this.routersConfig.forEach(x => {
            x.forEach(y => {
                if (y.name === name) {
                    result = y.path;
                }
            });
        });
        return (this.usehash ? '#' : '') + result;
    }
    registerAuthRouteHandler(callback) {
        this.routeAuthCallback = callback;
    }
    registerLoadingHandler(callback) {
        this.loadingHandlerCallback = callback;
    }
    registerRouteChangeHandler(callback) {
        this.routeChangeHandlerCallback = callback;
    }
    addRouterConfig(name, routes) {
        if (this.routers.indexOf(name) !== -1) {
            console.error('can not have 2 routers with same name');
        }
        else {
            let currentRoutes = [];
            routes.forEach(y => {
                if (this.href(y.name) !== (this.usehash ? '#' : '') + 'unknown') {
                    console.error('you should need to have unique names for routes, please fix:', name, y.name);
                }
                if (currentRoutes.indexOf(y.name) !== -1) {
                    console.error('you have same name on some of routes in new config, please fix:', y.name);
                }
                currentRoutes.push(y.name);
            });
            this.routers.push(name);
            this.routersConfig.push(routes);
        }
    }
    removeRouterConfig(name) {
        let i = this.routers.indexOf(name);
        if (i !== -1) {
            this.routers.splice(i, 1);
            this.routersConfig.splice(i, 1);
        }
    }
}
const router = new RouterInternal();
export function getRouter() {
    return router;
}
export function href(name, _router) {
    return router.href(name, _router);
}
export function navs(routerName) {
    return router.getNavLinks(routerName);
}
export function addRouterConfig(name, routes) {
    return router.addRouterConfig(name, routes);
}
export function removeRouterConfig(name) {
    return router.removeRouterConfig(name);
}
export function unknowRouteHandler(callback) {
    return router.registerUnknowRouteHandler(callback);
}
export function authRouteHandler(callback) {
    return router.registerAuthRouteHandler(callback);
}
export function goto(hash, params = {}) {
    return router.goto(hash, params);
}
//# sourceMappingURL=router.js.map