import { html } from 'lit-html';
import { addRouterConfig, removeRouterConfig, navs } from 'lit-html-free-router';
import { registerCustomElement } from '../litHtmlHelpers';


registerCustomElement(
    'else-component',
    class extends HTMLElement {
        routes: any[];
        router: any;


        activate(path: string) {
            console.log('------------------------------------new settings');
            addRouterConfig('sub', [
                {
                    name: 'home sub',
                    path: path,
                    load: () => {
                        return import('./home');
                    },
                    componentName: 'home-component'
                },
                {
                    name: 'else sub',
                    path: `${path}/about`,
                    load: () => {
                        return import('./about');
                    },
                    componentName: 'about-component'
                },
                {
                    name: 'settings sub',
                    path: `${path}/settings`,
                    load: () => {
                        return import('./settings');
                    },
                    componentName: 'settings-component'
                }
            ]);
            return Promise.resolve(true);
        }

        disconnectedCallback() {
            removeRouterConfig('sub');
        }

        public render() {

            return html`
                else
                <div class="nav">
                    ${navs('sub').map(route => {
                        if (route.isNav) {
                            return html`
                                <a href=${route.href} class="navitem">${route.name}</a>
                            `;
                        } else {
                            return '';
                        }
                    })}
                </div>
                <section>
                    <free-router name="sub"></free-router>
                </section>
            `;
        }
    }
);
