import { render } from 'lit-html';


export const registerCustomElement = (elementName: string, elementClass: any): void => {
    if (!customElements.get(elementName)) {
      const rendermethod = elementClass.prototype.render;
      const connectedCallbackMethod = elementClass.prototype.connectedCallback;
      const disconnectedCallbackMethod = elementClass.prototype.disconnectedCallback;
  
      elementClass.prototype.render = function (...result: any[]) {
        render(rendermethod.call(this, ...result), this, { eventContext: this });
      };
  
      elementClass.prototype.connectedCallback = function () {
   
        if (connectedCallbackMethod) {
          connectedCallbackMethod.call(this);
        }
  
        this.render();
      };
  
      elementClass.prototype.disconnectedCallback = function () {
  
        if (disconnectedCallbackMethod) {
          disconnectedCallbackMethod.call(this);
        }
  
      };
  
      customElements.define(elementName, elementClass);
    } else {
      console.log('element already registered');
    }
  };
  
  
  const singeltonMap = new Map();
  
  export const singelton = (_class: any) => {
    if(singeltonMap.has(_class)){
      return singeltonMap.get(_class)
    } else {
      const newclass = new _class();
      singeltonMap.set(_class, newclass);
      return newclass
    }
  }
  
  
  
  export class EventAggregator {
  
      private channels: any;
  
      constructor() {
          this.channels = {};
      }
  
      public publish(channel: string, ...args: any[]): void {
          if (Array.isArray(this.channels[channel])) {
              for (let i = 0, len = this.channels[channel].length; i < len; i++) {
                  this.channels[channel][i].func.apply(this, args);
              }
          } else {
              // if no one is listening then do we really need to do anything ?
          }
      }
  
      public unsubscribe(channel: string, ctx: any): void {
          if (Array.isArray(this.channels[channel])) {
              for (let i = 0, len = this.channels[channel].length; i < len; i++) {
                  if (this.channels[channel][i].ctx === ctx) {
                      this.channels[channel].splice(i, 1);
                  }
              }
          } else {
              // channel does not exist, great!
          }
  
      }
  
      public subscribe(channel: string, ctx: any, func: Function): void {
          if (!Array.isArray(this.channels[channel])) {
              this.channels[channel] = [];
          }
          this.channels[channel].push({ctx: ctx, func: func});
      }
  }